package cn.fw.shirasawa.common.utils;

import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GeodeticCalculator;
import org.gavaghan.geodesy.GeodeticCurve;
import org.gavaghan.geodesy.GlobalCoordinates;

/**
 * @author : kurisu
 * @className : TestUtils
 * @description :
 * @date: 2021-01-22 09:34
 */
public class GeoDistanceUtils {
    /**
     * @param poi      经度 中心点
     * @param distance 范围（米）
     * @return
     */
    public static Square calSquarePoint(Poi poi, double distance) {
        // 计算经度弧度,从弧度转换为角度
        double latitude = poi.getLatitude();
        double longitude = poi.getLongitude();
        double dLongitude = 2 * (Math.asin(Math.sin(distance
                / (2 * 6378137))
                / Math.cos(Math.toRadians(latitude))));
        dLongitude = Math.toDegrees(dLongitude);
        // 计算纬度角度
        double dLatitude = distance / 6378137;
        dLatitude = Math.toDegrees(dLatitude);
        // 正方形
        Poi leftTop = new Poi(longitude - dLongitude, latitude + dLatitude);
        Poi rightTop = new Poi(longitude + dLongitude, latitude + dLatitude);
        Poi leftBottom = new Poi(longitude - dLongitude, latitude - dLatitude);
        Poi rightBottom = new Poi(longitude + dLongitude, latitude - dLatitude);
        return new Square(leftTop, rightTop, leftBottom, rightBottom);
    }

    /**
     * 计算距离 百度坐标系的点
     *
     * @param source 第一个点
     * @param target 第二个点
     * @return
     */
    public static double getDistanceByBd09(Poi source, Poi target) {
        double[] w1 = CoordinateTransformUtil.bd09toWgs84(source.getLongitude(), source.getLatitude());
        double[] w2 = CoordinateTransformUtil.bd09toWgs84(target.getLongitude(), target.getLatitude());

        GlobalCoordinates begin = new GlobalCoordinates(w1[1], w1[0]);
        GlobalCoordinates end = new GlobalCoordinates(w2[1], w2[0]);
        GeodeticCurve curve = new GeodeticCalculator().calculateGeodeticCurve(Ellipsoid.WGS84, end, begin);
        return curve.getEllipsoidalDistance();
    }

    /**
     * 计算距离 火星坐标系的点（高德、谷歌）
     *
     * @param source 第一个点
     * @param target 第二个点
     * @return
     */
    public static double getDistanceByGcj02(Poi source, Poi target) {
        double[] w1 = CoordinateTransformUtil.gcj02toWgs84(source.getLongitude(), source.getLatitude());
        double[] w2 = CoordinateTransformUtil.gcj02toWgs84(target.getLongitude(), target.getLatitude());

        GlobalCoordinates begin = new GlobalCoordinates(w1[1], w1[0]);
        GlobalCoordinates end = new GlobalCoordinates(w2[1], w2[0]);
        GeodeticCurve curve = new GeodeticCalculator().calculateGeodeticCurve(Ellipsoid.WGS84, end, begin);
        return curve.getEllipsoidalDistance();
    }

    /**
     * 计算距离 wgs84坐标系的点
     *
     * @param source 第一个点
     * @param target 第二个点
     * @return
     */
    public static double getDistanceByWgs84(Poi source, Poi target) {
        GlobalCoordinates begin = new GlobalCoordinates(source.getLatitude(), source.getLongitude());
        GlobalCoordinates end = new GlobalCoordinates(target.getLatitude(), target.getLongitude());
        GeodeticCurve curve = new GeodeticCalculator().calculateGeodeticCurve(Ellipsoid.WGS84, end, begin);
        return curve.getEllipsoidalDistance();
    }


    public static void main(String[] args) {
        Square square = calSquarePoint(new Poi(116.404, 39.915), 1000);

//        System.out.println("leftTop：" + square.getLeftTopPoint().getLatitude() + "===" + square.getLeftTopPoint().getLongitude());
//        System.out.println("rightTop：" + square.getRightTopPoint().getLatitude() + "===" + square.getRightTopPoint().getLongitude());
//        System.out.println("leftBottom：" + square.getLeftBottomPoint().getLatitude() + "===" + square.getLeftBottomPoint().getLongitude());
//        System.out.println("rightBottom：" + square.getRightBottomPoint().getLatitude() + "===" + square.getRightBottomPoint().getLongitude());

        System.out.println(getDistanceByBd09(square.getLeftTopPoint(), square.getRightBottomPoint()));

    }

    public static class Square {
        /**
         * 左上的点
         */
        private Poi leftTopPoint;
        /**
         * 右上的点
         */
        private Poi rightTopPoint;
        /**
         * 左下的点
         */
        private Poi leftBottomPoint;
        /**
         * 右下的点
         */
        private Poi rightBottomPoint;

        public Square(Poi leftTopPoint, Poi rightTopPoint, Poi leftBottomPoint, Poi rightBottomPoint) {
            this.leftTopPoint = leftTopPoint;
            this.rightTopPoint = rightTopPoint;
            this.leftBottomPoint = leftBottomPoint;
            this.rightBottomPoint = rightBottomPoint;
        }

        public Poi getLeftTopPoint() {
            return leftTopPoint;
        }

        public void setLeftTopPoint(Poi leftTopPoint) {
            this.leftTopPoint = leftTopPoint;
        }

        public Poi getRightTopPoint() {
            return rightTopPoint;
        }

        public void setRightTopPoint(Poi rightTopPoint) {
            this.rightTopPoint = rightTopPoint;
        }

        public Poi getLeftBottomPoint() {
            return leftBottomPoint;
        }

        public void setLeftBottomPoint(Poi leftBottomPoint) {
            this.leftBottomPoint = leftBottomPoint;
        }

        public Poi getRightBottomPoint() {
            return rightBottomPoint;
        }

        public void setRightBottomPoint(Poi rightBottomPoint) {
            this.rightBottomPoint = rightBottomPoint;
        }
    }

    public static class Poi {
        /**
         * 经度
         */
        private double longitude;
        /**
         * 纬度
         */
        private double latitude;

        public Poi(double longitude, double latitude) {
            this.longitude = longitude;
            this.latitude = latitude;
        }

        public double getLongitude() {
            return longitude;
        }

        public void setLongitude(double longitude) {
            this.longitude = longitude;
        }

        public double getLatitude() {
            return latitude;
        }

        public void setLatitude(double latitude) {
            this.latitude = latitude;
        }
    }
}
