package cn.fw.shirasawa.rpc.valhalla;

import cn.fw.data.base.domain.common.Message;
import cn.fw.shirasawa.rpc.AbsBaseRpcService;
import cn.fw.shirasawa.rpc.valhalla.dto.ValhallaCustomerDTO;
import cn.fw.valhalla.sdk.api.CustomerApiService;
import cn.fw.valhalla.sdk.result.CustomerInfoDto;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Objects;

import static org.apache.commons.lang3.Validate.isTrue;

/**
 * 售后crm rpc
 *
 * @author : kurisu
 * @version : 1.0
 * @className : ValhallRpcService
 * @description : 售后crm rpc
 * @date : 2022-07-22 09:41
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ValhallaRpcService extends AbsBaseRpcService {

    private final CustomerApiService customerApiService;
    @Value("${spring.cache.custom.global-prefix}:rpc:valhalla")
    private String keyPrefix;

    @Override
    protected String getKeyPrefix() {
        return this.keyPrefix;
    }

    /**
     * 根据车架号查询售后客户档案
     *
     * @param vin     车架后
     * @param groupId 集团id
     * @return 会员信息
     */
    public ValhallaCustomerDTO queryByVin(final String vin, final Long groupId) {
        if (StringUtils.isBlank(vin)) {
            return null;
        }
        String key = String.format("%s:info:%s", getKeyPrefix(), vin);
        ValhallaCustomerDTO customerDTO = getFromCache(key, ValhallaCustomerDTO.class);
        if (Objects.nonNull(customerDTO)) {
            return customerDTO;
        }
        try {
            final Message<CustomerInfoDto> msg = customerApiService.queryByFrameNo(vin, groupId);
            isTrue(msg.isSuccess(), String.format("调用Valhalla系统失败: [根据vin[%s]获取档案信息]", vin));
            final CustomerInfoDto customer = msg.getData();
            if (Objects.isNull(customer)) {
                return null;
            }
            final ValhallaCustomerDTO info = new ValhallaCustomerDTO();
            BeanUtils.copyProperties(customer, info);
            setToCache(key, JSONObject.toJSONString(info), 60 * 5);
            return info;
        } catch (Exception e) {
            log.error("调用Valhalla系统根据vin[{}]获取保有客信息失败，原因：{}", vin, e);
        }
        return null;
    }
}
