package cn.fw.shirasawa.server.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.shirasawa.domain.dto.FollowAttachmentDTO;
import cn.fw.shirasawa.domain.vo.follow.ClueHistoryVO;
import cn.fw.shirasawa.domain.vo.follow.DateFollowRecordHistoryVO;
import cn.fw.shirasawa.domain.vo.follow.FollowDetailVO;
import cn.fw.shirasawa.domain.vo.follow.FollowRecordHistoryVO;
import cn.fw.shirasawa.service.bus.follow.FollowBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @title 跟进控制器
 * @className : FollowController
 * @description : 跟进控制器
 * @date: 2020-08-15 16:23
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/follow")
public class FollowController {
    private final FollowBizService followBizService;

    @Autowired
    public FollowController(final FollowBizService followBizService) {
        this.followBizService = followBizService;
    }

    @IgnoreAuth
    @GetMapping("/todo/detail")
    @ControllerMethod("查询跟进待办详情")
    public Message<FollowDetailVO> todoDetail(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        return success(followBizService.queryFollowDetail(recordId));
    }

    @GetMapping("/approve/detail")
    @IgnoreAuth
    @ControllerMethod("查询跟进战败审批详情")
    public Message<FollowDetailVO> approveDetail(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        FollowDetailVO detailVO = followBizService.queryApplyFollowDetail(recordId);
        return success(detailVO);
    }

    @IgnoreAuth
    @GetMapping("/todo/history")
    @ControllerMethod("查询跟进待办历史记录")
    public Message<List<FollowRecordHistoryVO>> todoRecordHistory(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        return success(followBizService.todoRecordHistory(recordId));
    }

    @GetMapping("/sms/template")
    @IgnoreAuth
    @ControllerMethod("查询短信模板")
    public Message<String> getSmsTemplate(@CurrentUser LoginAuthBean currentUser, @NotNull(message = "跟进类型不能为空") final Integer type) {
        Message<String> message = success();
        message.setData(followBizService.smsTemplate(currentUser.getUserId(), type));
        return message;
    }

    /**
     * 主动战败
     *
     * @param currentUser 当前登录用户
     * @param recordId
     * @param reason
     * @return
     */
    @PostMapping("/defeat")
    @ControllerMethod("主动战败")
    public Message<Void> defeat(@CurrentUser LoginAuthBean currentUser,
                                @NotNull(message = "跟进记录id不能为空") final Long recordId,
                                final String reason) {
        followBizService.defeat(currentUser, recordId, reason);
        return success();
    }

    /**
     * showdoc
     * 跟进记录id不能为空
     *
     * @param userId
     * @param recordId
     * @return
     */
    @PostMapping("/todo/complete")
    @ControllerMethod("完成跟进待办")
    public Message<Void> complete(@CurrentUser Long userId,
                                  @NotNull(message = "跟进记录id不能为空") final Long recordId) {
        followBizService.completeRecord(recordId, userId);
        return success();
    }

    /**
     * 上传跟进凭证
     *
     * @param userId
     * @param dto
     * @return
     */
    @PostMapping("/todo/upload")
    @ControllerMethod("上传跟进凭证")
    public Message<Void> upload(@CurrentUser Long userId, @RequestBody @Valid final FollowAttachmentDTO dto) {
        followBizService.uploadAtt(dto, userId);
        return success();
    }

    /**
     * 查询档案跟进线索历史列表
     *
     * @param currentUser
     * @param customerId
     * @param bizType
     * @return
     */
    @GetMapping("/clue/history/list")
    @IgnoreAuth
    @ControllerMethod("查询档案跟进线索历史列表")
    public Message<List<ClueHistoryVO>> clueHistory(@CurrentUser LoginAuthBean currentUser,
                                                    @NotNull(message = "档案id不能为空") final Long customerId,
                                                    final Integer bizType
    ) {
        List<ClueHistoryVO> list = followBizService.clueHistoryList(currentUser.getGroupId(), customerId, bizType);
        return success(list);
    }

    /**
     * 查询档案跟进线索历史列表
     *
     * @param currentUser
     * @param vin
     * @return
     */
    @GetMapping("/clue/after/history/list")
    @IgnoreAuth
    @ControllerMethod("查询vin跟进线索历史列表")
    public Message<List<ClueHistoryVO>> asClueHistory(@CurrentUser LoginAuthBean currentUser,
                                                      @NotBlank(message = "vin不能为空") final String vin
    ) {
        List<ClueHistoryVO> list = followBizService.asClueHistoryList(currentUser.getGroupId(), vin);
        return success(list);
    }

    /**
     * @param clueId 线索id
     * @return
     * @title 查询跟进线索的跟进历史记录
     */
    @GetMapping("/clue/history/record")
    @IgnoreAuth
    @ControllerMethod("查询跟进线索的跟进历史记录")
    public Message<List<FollowRecordHistoryVO>> clueRecord(@NotNull(message = "线索id不能为空") final Long clueId) {
        return success(followBizService.queryHistoryByClueId(clueId));
    }

    /**
     * @param recordId 跟进记录id
     * @return
     * @title 通过智能通话拨打电话
     */
    @GetMapping("/call")
    @ControllerMethod("通过智能通话拨打电话")
    public Message<String> call(@NotNull(message = "跟进待办ID不能为空") final Long recordId) {
        return success(followBizService.callSomeone(recordId));
    }

    /**
     * @param recordId 必选 string 跟进id
     * @return {"code":0,"result":"查询跟进记录详情成功","data":[{"ymd":"2022-05-30","followRecordLogList":[{"id":577368,"recordId":77094,"userName":"李志恒","taskId":80515,"attType":3,"followType":7,"feedbackType":1,"attachments":"7e9d150772b24db789e49607bdb3df7f","uploadTime":1653883201000}]}],"success":true}
     * @catalog APP跟进记录
     * @title 根据recordId获取跟进记录详细信息
     * @description 根据recordId获取跟进记录详细信息
     * @method get
     */
    @GetMapping("/followRecord/history/record")
    @IgnoreAuth
    @ControllerMethod("查询跟进记录详情")
    public Message<List<DateFollowRecordHistoryVO>> followRecordDetail(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        List<DateFollowRecordHistoryVO> detailVoList = followBizService.followRecordDetail(recordId);
        return success(detailVoList);
    }

    @GetMapping("/follow_record/log/detail")
    @IgnoreAuth
    @ControllerMethod("查询跟进记录池详情")
    public Message<List<FollowRecordHistoryVO>> recordPoolDetail(@NotNull(message = "跟进待办id不能为空") final Long id) {
        return success(followBizService.recordPoolDetail(id));
    }
}
