package cn.fw.shirasawa.server.controller.task;

import cn.fw.shirasawa.common.utils.StringUtils;
import cn.fw.shirasawa.domain.dto.CallReportDTO;
import cn.fw.shirasawa.rpc.ehr.EhrRpcService;
import cn.fw.shirasawa.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.shirasawa.rpc.member.MemberRpcService;
import cn.fw.shirasawa.rpc.member.dto.MemberUserDTO;
import cn.fw.shirasawa.service.bus.follow.FollowBizService;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : CallReportDealTask
 * @description : 通话记录处理任务
 * @date: 2021-01-19 16:02
 */
@Slf4j
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class CallReportDealTask {
    private final FollowBizService followBizService;
    private final EhrRpcService ehrRpcService;
    private final StringRedisTemplate redisTemplate;
    private final MemberRpcService memberRpcService;

    @Value("${spring.cache.custom.global-prefix}:mq:call:report")
    @Getter
    private String callReportKey;

    @Value("${spring.cache.custom.global-prefix}:mq:call:fail-report")
    @Getter
    private String failReportKey;

    @Autowired
    public CallReportDealTask(final FollowBizService followBizService,
                              final EhrRpcService ehrRpcService,
                              final StringRedisTemplate redisTemplate,
                              final MemberRpcService memberRpcService) {
        this.followBizService = followBizService;
        this.ehrRpcService = ehrRpcService;
        this.redisTemplate = redisTemplate;
        this.memberRpcService = memberRpcService;
    }

    /**
     * 处理通话记录
     */
    @Scheduled(initialDelay = 1000 * 5, fixedRate = 1000 * 15)
    public void dealCallReport() {
        List<String> failList = new ArrayList<>();
        String callStr;
        while ((callStr = redisTemplate.opsForList().leftPop(getCallReportKey())) != null) {
            CallReportDTO dto = JSONObject.parseObject(callStr, CallReportDTO.class);
            if (Objects.isNull(dto)) {
                continue;
            }
            if (dto.getTalkTime() <= 0) {
                continue;
            }
            final String staffMobile = dto.getStaffMobile();
            final String peerMobile = dto.getPeerNo();
            final Long groupId = dto.getGroupId();
            final Long staffId = dto.getStaffId();
            try {
                boolean isStaffValid = Objects.nonNull(staffId) && Objects.nonNull(groupId);
                if (!isStaffValid) {
                    StaffInfoDTO info = ehrRpcService.queryStaffInfoByMobile(staffMobile);
                    if (Objects.isNull(info) && dto.getCallTime().isBefore(LocalDateTime.now().minusMinutes(20L))) {
                        redisTemplate.opsForList().rightPush(getFailReportKey(), callStr);
                        continue;
                    }
                    BV.notNull(info, () -> String.format("[%s]员工信息获取失败", staffMobile));
                    dto.setStaffId(info.getId());
                    dto.setStaffName(info.getName());
                    dto.setGroupId(info.getGroupId());
                }
                if (Objects.isNull(dto.getMemberId())) {
                    if (dto.getCallTime().isBefore(LocalDateTime.now().minusMinutes(20L))) {
                        redisTemplate.opsForList().rightPush(getFailReportKey(), callStr);
                        continue;
                    }
                    MemberUserDTO memberInfo = memberRpcService.queryByMobile(peerMobile);
                    if (Objects.nonNull(memberInfo)) {
                        dto.setMemberId(memberInfo.getUserId());
                        dto.setMemberName(memberInfo.getRealName());
                    }
                }
                followBizService.readCallReport(dto);
            } catch (Exception e) {
                if (StringUtils.isValid(callStr)) {
                    failList.add(callStr);
                }
                log.error("处理通话记录失败: [{}]", e.getMessage());
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            redisTemplate.opsForList().rightPushAll(getCallReportKey(), failList);
        }
    }

}
