package cn.fw.shirasawa.service.bus.follow;

import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.domain.db.OriginalData;
import cn.fw.shirasawa.domain.dto.OriginalDataDTO;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import cn.fw.shirasawa.service.data.OriginalDataService;
import cn.fw.shirasawa.service.event.TaskCancelEvent;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : OriginalDataBizService
 * @description : 跟进原始数据处理服务
 * @date: 2020-08-14 17:47
 */
@Slf4j
@Service
public class OriginalDataBizService {
    private final OriginalDataService originalDataService;
    private final ApplicationEventPublisher eventPublisher;

    @Autowired
    public OriginalDataBizService(final OriginalDataService originalDataService,
                                  final ApplicationEventPublisher eventPublisher) {
        this.originalDataService = originalDataService;
        this.eventPublisher = eventPublisher;
    }

    /**
     * 保存原始数据
     *
     * @param dataDTO
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveOrigin(OriginalDataDTO dataDTO) {
        OriginalData originalData = new OriginalData();
        BeanUtils.copyProperties(dataDTO, originalData);
        originalData.setSolved(Boolean.FALSE);
        originalData.setType(FollowTypeEnum.ofValue(dataDTO.getDataType()));
        Map<String, String> noteMap = dataDTO.getNoteMap();
        if (Objects.isNull(dataDTO.getMemberId())) {
            originalData.setMemberId(-1L);
        }
        originalData.setGenerateTime(DateUtil.date2LocalDateTime(dataDTO.getGenerateTime()));
        originalData.setDeadline(DateUtil.date2LocalDateTime(dataDTO.getDeadline()));
        originalData.setNote(JSONObject.toJSONString(noteMap));
        originalData.setFirstDeadline(DateUtil.date2LocalDateTime(dataDTO.getFirstRecordDeadline()));
        originalDataService.save(originalData);
    }

    /**
     * 保存原始数据
     *
     * @param list
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveBatchOrigin(List<OriginalDataDTO> list) {
        List<OriginalData> dbList = new ArrayList<>();
        for (OriginalDataDTO dataDTO : list) {
            OriginalData originalData = new OriginalData();
            BeanUtils.copyProperties(dataDTO, originalData);
            Map<String, String> noteMap = dataDTO.getNoteMap();
            originalData.setGenerateTime(DateUtil.date2LocalDateTime(dataDTO.getGenerateTime()));
            originalData.setDeadline(DateUtil.date2LocalDateTime(dataDTO.getDeadline()));
            originalData.setNote(JSONObject.toJSONString(noteMap));
            originalData.setSolved(Boolean.FALSE);
            originalData.setType(FollowTypeEnum.ofValue(dataDTO.getDataType()));
            originalData.setFirstDeadline(DateUtil.date2LocalDateTime(dataDTO.getFirstRecordDeadline()));
            dbList.add(originalData);
        }
        originalDataService.saveBatch(dbList);
    }

    /**
     * 删除无效原始数据
     *
     * @param type
     * @param detailId
     */
    @Transactional(rollbackFor = Exception.class)
    public void remove(DataTypeEnum type, String detailId) {
        OriginalData data = originalDataService.getOne(Wrappers.<OriginalData>lambdaQuery()
                .eq(OriginalData::getDetailId, detailId)
                .eq(OriginalData::getType, type)
        );
        if (Objects.nonNull(data)) {
            originalDataService.removeById(data.getId());
            if (Boolean.TRUE.equals(data.getSolved())) {
                TaskCancelEvent event = new TaskCancelEvent(type, data.getCustomerId(), data.getGenerateTime(), data.getGroupId());
                eventPublisher.publishEvent(event);
            }
        }
    }

    /**
     * 修改原始数据标识
     *
     * @param id
     */
    @Transactional(rollbackFor = Exception.class)
    public void solve(Long id) {
        originalDataService.update(Wrappers.<OriginalData>lambdaUpdate()
                .set(OriginalData::getSolved, Boolean.TRUE)
                .eq(OriginalData::getId, id)
        );
    }
}
