package cn.fw.shirasawa.service.bus.follow.strategy;

import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.common.utils.MobileUtil;
import cn.fw.shirasawa.common.utils.StringUtils;
import cn.fw.shirasawa.domain.db.OriginalData;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowRecordLog;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.pool.CluePool;
import cn.fw.shirasawa.domain.dto.FollowAttachmentDTO;
import cn.fw.shirasawa.domain.dto.FollowTaskCompleteDTO;
import cn.fw.shirasawa.domain.dto.FollowTerminationDTO;
import cn.fw.shirasawa.domain.enums.*;
import cn.fw.shirasawa.domain.vo.follow.FollowDetailVO;
import cn.fw.shirasawa.domain.vo.follow.FollowRecordHistoryVO;
import cn.fw.shirasawa.rpc.backlog.TodoRpcService;
import cn.fw.shirasawa.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.shirasawa.rpc.ehr.EhrRpcService;
import cn.fw.shirasawa.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.shirasawa.rpc.erp.UserService;
import cn.fw.shirasawa.rpc.erp.dto.UserInfoDTO;
import cn.fw.shirasawa.rpc.member.MemberRpcService;
import cn.fw.shirasawa.rpc.member.dto.MemberUserDTO;
import cn.fw.shirasawa.rpc.oop.OopService;
import cn.fw.shirasawa.rpc.oop.dto.ShopDTO;
import cn.fw.shirasawa.rpc.pstn.PstnRpcService;
import cn.fw.shirasawa.sdk.enums.BusinessTypeEnum;
import cn.fw.shirasawa.service.data.*;
import cn.fw.shirasawa.service.event.CancelApproveEvent;
import cn.fw.shirasawa.service.event.RecordCompleteEvent;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : AbstractFollowStrategy
 * @description : 策略抽象类
 * @date: 2020-08-17 10:42
 */
@Slf4j
@SuppressWarnings("Duplicates")
public abstract class AbstractFollowStrategy implements FollowStrategy {
    @Autowired
    protected FollowTaskService followTaskService;
    @Autowired
    protected FollowRecordService followRecordService;
    @Autowired
    protected UserService userService;
    @Autowired
    protected FollowRecordLogService followRecordLogService;
    @Autowired
    protected OopService oopService;
    @Autowired
    protected ApplicationEventPublisher eventPublisher;
    @Autowired
    protected TodoRpcService todoRpcService;
    @Autowired
    protected CluePoolService cluePoolService;
    @Autowired
    protected StringRedisTemplate redisTemplate;
    @Autowired
    protected EhrRpcService ehrRpcService;
    @Autowired
    protected PstnRpcService pstnRpcService;
    @Autowired
    protected SecretReportHistoryService secretReportHistoryService;
    @Autowired
    protected MemberRpcService memberRpcService;


    @Value("${spring.cache.locker.key-prefix}:termination")
    @Getter
    private String planKey;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void overdue(FollowRecord record) {
        record.setOutTime(OutTimeEnum.BE_OVERDUE);
        if (!Boolean.TRUE.equals(record.getAddTodo())) {
            record.setYn(Boolean.FALSE);
            followRecordService.updateById(record);
        } else {
            followRecordService.overdue(record.getId());
        }
        FollowTask task = followTaskService.getById(record.getTaskId());
        eventPublisher.publishEvent(new RecordCompleteEvent(record, task.getClueId()));
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void complete(FollowRecord record, Long userId) {
        FollowTask task = followTaskService.getById(record.getTaskId());
        BV.isTrue(record.getUserId().equals(userId), () -> "无法完成非本人待办任务");
        BV.isTrue(OutTimeEnum.ONGOING.equals(record.getOutTime()), () -> "无法完成已逾期的待办任务");
        List<FollowRecordLog> list = followRecordLogService.list(Wrappers.<FollowRecordLog>lambdaQuery()
                .eq(FollowRecordLog::getRecordId, record.getId())
        );
        BV.isNotEmpty(list, () -> "请上传跟进记录");

        boolean match = list.stream().allMatch(log -> Objects.nonNull(log.getFeedbackType()));
        BV.isTrue(match, () -> "请检查跟进记录是否已完成客户反馈的录入");

        if (Boolean.TRUE.equals(record.getCompelTel())) {
            boolean matchAttType = list.stream().anyMatch(log -> AttTypeEnum.SMART_PHONE == log.getAttType());
            BV.isTrue(matchAttType, () -> "您本次没有拨打电话，无法完成跟进记录！");
        }

        record.setFollowTime(LocalDateTime.now());
        record.setOutTime(OutTimeEnum.BE_COMPLETE);
        followRecordService.updateById(record);
        if (Objects.nonNull(task)) {
            int times = task.getTimes() == null ? 0 : task.getTimes();
            task.setTimes(times + 1);
            followTaskService.updateById(task);
        }
        BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), String.valueOf(record.getId()),
                DateUtil.toDate(record.getFollowTime()), record.getShopId());
        todoRpcService.complete(dto);

        eventPublisher.publishEvent(new RecordCompleteEvent(record, task.getClueId()));
    }

    /**
     * 与上面代码重复 考虑到出错可能性 暂不对上面代码改造
     *
     * @param record      跟进待办
     * @param userId      完成人
     * @param ignoreCheck 是否忽略检查  true-忽略  false-不忽略
     */
    @Override
    public void complete(FollowRecord record, Long userId, Boolean ignoreCheck) {
        FollowTask task = followTaskService.getById(record.getTaskId());

        if (!ignoreCheck) {
            BV.isTrue(record.getUserId().equals(userId), () -> "无法完成非本人待办任务");
            BV.isTrue(OutTimeEnum.ONGOING.equals(record.getOutTime()), () -> "无法完成已逾期的待办任务");
            List<FollowRecordLog> list = followRecordLogService.list(Wrappers.<FollowRecordLog>lambdaQuery()
                    .eq(FollowRecordLog::getRecordId, record.getId())
            );
            BV.isNotEmpty(list, () -> "请上传跟进记录");
            boolean match = list.stream().allMatch(log -> Objects.nonNull(log.getFeedbackType()));
            BV.isTrue(match, () -> "请检查跟进记录是否已完成客户反馈的录入");
        } else {
            record.setOutTime(OutTimeEnum.BE_SYSTEM);
        }
        record.setFollowTime(LocalDateTime.now());

        followRecordService.updateById(record);
        if (Objects.nonNull(task)) {
            int times = task.getTimes() == null ? 0 : task.getTimes();
            task.setTimes(times + 1);
        }
        followTaskService.updateById(task);
        BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), String.valueOf(record.getId()),
                DateUtil.toDate(record.getFollowTime()), record.getShopId());
        todoRpcService.complete(dto);
        eventPublisher.publishEvent(new RecordCompleteEvent(record, task.getClueId()));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createNewRecord(FollowTask task, LocalDateTime nextTime, LocalDateTime nextEndTime, String note) {
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void completeTask(FollowTaskCompleteDTO taskCompleteDTO) {
        List<CluePool> cluePoolList = cluePoolService.list(Wrappers.<CluePool>lambdaQuery()
                .eq(CluePool::getRefererId, taskCompleteDTO.getDetailId())
                .eq(CluePool::getClueType, taskCompleteDTO.getFollowType())
                .in(CluePool::getClueStatus, ClueStatusEnum.WAITING, ClueStatusEnum.ONGOING)
                .eq(CluePool::getBizType, taskCompleteDTO.getBizType())
        );
        if (CollectionUtils.isEmpty(cluePoolList)) {
            return;
        }
        for (CluePool cluePool : cluePoolList) {
            cluePool.setFinishShopId(taskCompleteDTO.getShopId());
            cluePool.setFinishUserId(taskCompleteDTO.getUserId());
            UserInfoDTO user = userService.user(taskCompleteDTO.getUserId());
            if (Objects.nonNull(user)) {
                cluePool.setFinishUserName(user.getUserName());
            }
            ShopDTO shop = oopService.shop(taskCompleteDTO.getShopId());
            if (Objects.nonNull(shop)) {
                cluePool.setFinishShopName(shop.getShortName());
            }
            cluePool.setCloseTime(DateUtil.date2LocalDateTime(taskCompleteDTO.getCompleteTime()));
            if (ClueStatusEnum.ONGOING.equals(cluePool.getClueStatus())) {
                FollowTask task = followTaskService.queryOngoingTaskByClueId(cluePool.getId());
                if (Objects.isNull(task)) {
                    return;
                }
                task.setFinishUser(cluePool.getFinishUserId());
                task.setFinishUserName(cluePool.getFinishUserName());
                task.setFinishShop(cluePool.getFinishShopId());
                task.setCloseTime(cluePool.getCloseTime());
                boolean equals = task.getFollowShop().equals(cluePool.getFinishShopId());
                task.setState(equals ? TaskStateEnum.COMPLETE : TaskStateEnum.DEFEAT);
                if (!equals) {
                    task.setReason(TaskDefeatTypeEnum.F);
                }
                boolean succeed = followTaskService.updateById(task);
                if (succeed) {
                    completeTodo(task.getId());
                    CancelApproveEvent event = new CancelApproveEvent(task.getId(), ApproveTypeEnum.FOLLOW_DEFEAT);
                    eventPublisher.publishEvent(event);
                }
            }
            cluePool.setClueStatus(ClueStatusEnum.COMPLETE);
            cluePoolService.updateById(cluePool);
        }
    }

    @Override
    public void terminationTaskPlan(FollowTerminationDTO terminationDTO) {
        List<CluePool> cluePoolList = cluePoolService.list(Wrappers.<CluePool>lambdaQuery()
                .eq(CluePool::getRefererId, terminationDTO.getDetailId())
                .eq(CluePool::getClueType, terminationDTO.getFollowType())
                .in(CluePool::getClueStatus, ClueStatusEnum.WAITING, ClueStatusEnum.ONGOING)
                .eq(CluePool::getBizType, terminationDTO.getBizType())
        );
        if (CollectionUtils.isEmpty(cluePoolList)) {
            return;
        }
        for (CluePool cluePool : cluePoolList) {
            endTask(cluePool);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void endTask(CluePool cluePool) {
        if (ClueStatusEnum.WAITING.equals(cluePool.getClueStatus())) {
            cluePool.setCloseTime(LocalDateTime.now());
            cluePool.setClueStatus(ClueStatusEnum.FAILURE);
            cluePoolService.updateById(cluePool);
        }

        FollowTask task = followTaskService.queryOngoingTaskByClueId(cluePool.getId());
        if (Objects.nonNull(task)) {
            task.setCloseTime(LocalDateTime.now());
            task.setReason(TaskDefeatTypeEnum.E);
            endTask(task, false);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void endTask(FollowTask task, boolean fromFlow) {
        Long clueId = task.getClueId();
        if (TaskStateEnum.ONGOING == task.getState()) {
            task.setState(TaskStateEnum.DEFEAT);
            task.setCloseTime(Objects.isNull(task.getCloseTime()) ? task.getDeadline() : task.getCloseTime());
            if (Objects.isNull(task.getReason())) {
                task.setReason(TaskDefeatTypeEnum.C);
            }
            followTaskService.updateById(task);
            completeTodo(task.getId());
        }
        CluePool cluePool = cluePoolService.getById(clueId);
        cluePool.setCloseTime(task.getCloseTime());
        cluePool.setClueStatus(ClueStatusEnum.FAILURE);
        cluePoolService.updateById(cluePool);

        if (!fromFlow) {
            CancelApproveEvent event = new CancelApproveEvent(task.getId(), ApproveTypeEnum.FOLLOW_DEFEAT);
            eventPublisher.publishEvent(event);
        }
    }

    @Override
    public FollowDetailVO queryDetail(FollowRecord record) {
        FollowTask followTask = followTaskService.getById(record.getTaskId());
        BV.notNull(followTask, "跟进任务不存在");
        CluePool cluePool = cluePoolService.getById(followTask.getClueId());
        BV.notNull(cluePool, "跟进线索不存在");

        FollowDetailVO vo = new FollowDetailVO();
        assembleRecordDetail(vo, record, cluePool);
        vo.setTaskDeadline(followTask.getDeadline());
        return vo;
    }

    @Override
    public List<FollowRecordHistoryVO> queryHistoryByRecord(FollowRecord record) {
        List<FollowRecordHistoryVO> list = new ArrayList<>();
        if (BusinessTypeEnum.AS.equals(record.getBizType()) && !FollowTypeEnum.AC.equals(record.getType())) {
            List<FollowTask> taskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                    .eq(FollowTask::getType, record.getType())
                    .eq(FollowTask::getBizId, record.getBizId())
                    .eq(FollowTask::getGroupId, record.getGroupId())
                    .ge(FollowTask::getCloseTime, LocalDateTime.now().minusMonths(2L))
                    .ne(FollowTask::getId, record.getTaskId())
            );
            if (!CollectionUtils.isEmpty(taskList)) {
                for (FollowTask followTask : taskList) {
                    list.addAll(queryHistoryByTask(followTask));
                }
            }
        }
        FollowTask task = followTaskService.getById(record.getTaskId());
        list.addAll(queryHistoryByTask(task));
        return list.stream().sorted((a, b) -> b.getId().compareTo(a.getId())).collect(Collectors.toList());
    }

    @Override
    public List<FollowRecordHistoryVO> queryHistoryByTask(FollowTask task) {
        List<FollowRecord> followRecordList = new ArrayList<>();
        if (Objects.nonNull(task)) {
            Long clueId = task.getClueId();
            List<FollowTask> taskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                    .eq(FollowTask::getClueId, clueId)
                    .ne(FollowTask::getId, task.getId())
            );
            if (!CollectionUtils.isEmpty(taskList)) {
                for (FollowTask followTask : taskList) {
                    List<FollowRecord> recordList = followRecordService.getRecordList(followTask.getId());
                    if (!CollectionUtils.isEmpty(recordList)) {
                        followRecordList.addAll(recordList);
                    }
                }
            }
        }
        List<FollowRecord> recordList = followRecordService.getRecordList(task.getId());
        if (!CollectionUtils.isEmpty(recordList)) {
            followRecordList.addAll(recordList);
        }
        if (CollectionUtils.isEmpty(followRecordList)) {
            return new ArrayList<>();
        }
        List<Long> recordIds = followRecordList.stream().map(FollowRecord::getId).collect(Collectors.toList());
        Map<Long, String> map = followRecordList.stream().collect(Collectors.toMap(FollowRecord::getId, FollowRecord::getUserName));
        List<FollowRecordLog> attachments = followRecordLogService.getListByRecordIds(recordIds);
        if (CollectionUtils.isEmpty(attachments)) {
            return new ArrayList<>();
        }
        MemberUserDTO user = memberRpcService.user(task.getMemberId());
        List<FollowRecordHistoryVO> list = new ArrayList<>();
        for (FollowRecordLog attachment : attachments) {
            FollowRecordHistoryVO vo = new FollowRecordHistoryVO();
            vo.setAttachments(attachment.getAttachments());
            vo.setId(attachment.getId());
            vo.setCustomerName(Optional.ofNullable(user).map(MemberUserDTO::getRealName).orElse(null));
            vo.setRecordId(attachment.getRecordId());
            vo.setTaskId(attachment.getTaskId());
            vo.setAttType(attachment.getAttType());
            vo.setFeedbackType(attachment.getFeedbackType());
            vo.setFollowType(task.getType());
            vo.setUploadTime(attachment.getUploadTime());
            vo.setDescribes(attachment.getDescribes());
            vo.setUserName(map.get(attachment.getRecordId()));
            list.add(vo);
        }
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void uploadAtt(FollowAttachmentDTO dto, FollowRecord record, Long userId) {
        BV.isTrue(record.getUserId().equals(userId), () -> "无法跟进非本人待办任务");
        BV.isTrue(OutTimeEnum.ONGOING.equals(record.getOutTime()), () -> "无法跟进已逾期的待办任务");
        FollowRecordLog recordLog = new FollowRecordLog();
        recordLog.setId(dto.getId());
        recordLog.setTaskId(record.getTaskId());
        recordLog.setAttachments(dto.getAttachments());
        recordLog.setFeedbackType(dto.getFeedbackTypeEnum());
        recordLog.setAttType(dto.getAttTypeEnum());
        recordLog.setDescribes(dto.getDescribes());
        recordLog.setRecordId(dto.getRecordId());
        if (Objects.isNull(dto.getId())) {
            recordLog.setUploadTime(LocalDateTime.now());
        }
        followRecordLogService.saveOrUpdate(recordLog);
        feedbackTask(record.getTaskId(), dto.getFeedbackTypeEnum());
    }

    @Override
    public List<FollowRecordHistoryVO> getRecordListByClue(CluePool cluePool) {
        Long clueId = cluePool.getId();
        List<FollowRecord> followRecordList = new ArrayList<>();
        List<FollowTask> taskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getClueId, clueId)
        );
        if (!CollectionUtils.isEmpty(taskList)) {
            for (FollowTask followTask : taskList) {
                List<FollowRecord> recordList = followRecordService.getRecordList(followTask.getId());
                if (!CollectionUtils.isEmpty(recordList)) {
                    followRecordList.addAll(recordList);
                }
            }
        }

        if (CollectionUtils.isEmpty(followRecordList)) {
            return new ArrayList<>();
        }
        List<Long> recordIds = followRecordList.stream().map(FollowRecord::getId).collect(Collectors.toList());
        Map<Long, FollowRecord> map = followRecordList.stream().collect(Collectors.toMap(FollowRecord::getId, record -> record));
        List<FollowRecordLog> attachments = followRecordLogService.getListByRecordIds(recordIds);
        if (CollectionUtils.isEmpty(attachments)) {
            return new ArrayList<>();
        }
        List<FollowRecordHistoryVO> list = new ArrayList<>();
        for (FollowRecordLog attachment : attachments) {
            FollowRecordHistoryVO vo = new FollowRecordHistoryVO();
            vo.setAttachments(attachment.getAttachments());
            vo.setId(attachment.getId());
            vo.setRecordId(attachment.getRecordId());
            vo.setTaskId(map.get(attachment.getRecordId()).getTaskId());
            vo.setAttType(attachment.getAttType());
            vo.setFeedbackType(attachment.getFeedbackType());
            vo.setFollowType(cluePool.getClueType());
            vo.setUploadTime(attachment.getUploadTime());
            vo.setDescribes(attachment.getDescribes());
            vo.setUserName(map.get(attachment.getRecordId()).getUserName());
            list.add(vo);
        }
        return list;
    }

    @Override
    public FollowDetailVO followPoolDetail(FollowTask task) {
        CluePool cluePool = cluePoolService.getById(task.getClueId());
        BV.notNull(cluePool, "跟进线索不存在");
        FollowDetailVO vo = new FollowDetailVO();
        vo.setCustomerId(task.getCustomerId());
        vo.setType(task.getType().getValue());
        vo.setBizType(cluePool.getBizType().getValue());
        vo.setTaskId(task.getId());
        vo.setDeadline(task.getDeadline());
        queryTimes(cluePool.getId(), vo);
        return vo;
    }

    /**
     * 原始数据转线索
     *
     * @param originalData
     * @return
     */
    protected CluePool transformClue(OriginalData originalData) {
        CluePool cluePool = new CluePool();
        cluePool.setRefererId(originalData.getDetailId());
        cluePool.setCustomerId(originalData.getCustomerId());
        cluePool.setCustomerName(originalData.getCustomerName());
        cluePool.setNote(originalData.getNote());
        cluePool.setContacts(originalData.getContacts());
        cluePool.setFrameNo(originalData.getFrameNo());
        cluePool.setPlateNo(originalData.getPlateNo());
        cluePool.setClueType(originalData.getType());
        cluePool.setBizType(originalData.getBizType());
        cluePool.setAddTime(originalData.getGenerateTime());
        cluePool.setClueStatus(ClueStatusEnum.WAITING);
        cluePool.setRedistribution(originalData.getSecondary());
        cluePool.setOriginalUserId(originalData.getUserId());
        cluePool.setOriginalUserName(originalData.getUserName());
        if (StringUtils.isEmpty(originalData.getUserName())) {
            StaffInfoDTO info = ehrRpcService.queryStaffInfo(originalData.getUserId());
            cluePool.setOriginalUserName(Optional.ofNullable(info).map(StaffInfoDTO::getName).orElse(null));
        }
        cluePool.setOriginalShopId(originalData.getShopId());
        cluePool.setOriginalShopName(originalData.getShopName());
        if (StringUtils.isEmpty(originalData.getShopName())) {
            ShopDTO shop = oopService.shop(originalData.getShopId());
            cluePool.setOriginalShopName(Optional.ofNullable(shop).map(ShopDTO::getShortName).orElse(null));
        }
        cluePool.setGroupId(originalData.getGroupId());
        cluePool.setMemberId(originalData.getMemberId());
        cluePool.setFirstDeadline(originalData.getFirstDeadline());
        cluePool.setBizId(originalData.getBizId());
        cluePool.setCoverFlag(originalData.getCoverFlag());
        return cluePool;
    }

    /**
     * 创建跟进任务
     *
     * @param cluePool
     * @return
     */
    protected FollowTask createTask(CluePool cluePool) {
        FollowTask task = new FollowTask();
        String customerName = cluePool.getCustomerName();
        MemberUserDTO user = memberRpcService.user(cluePool.getMemberId());
        if (Objects.nonNull(user)) {
            customerName = StringUtils.isValid(user.getRealName()) ? user.getRealName() : user.getNickName();
            if (StringUtils.isEmpty(cluePool.getContacts())) {
                cluePool.setContacts(user.getPhone());
            }
        }
        task.setClueId(cluePool.getId());
        task.setCustomerId(cluePool.getCustomerId());
        task.setMemberId(cluePool.getMemberId());
        task.setCustomerName(customerName);
        task.setPlateNo(cluePool.getPlateNo());
        task.setType(cluePool.getClueType());
        task.setBizType(cluePool.getBizType());
        task.setBeginTime(cluePool.getStartTime());
        task.setRedistribution(cluePool.getRedistribution());
        task.setDeadline(cluePool.getDeadline());
        task.setState(TaskStateEnum.ONGOING);
        task.setFollowUser(cluePool.getOriginalUserId());
        task.setFollowUserName(cluePool.getOriginalUserName());
        task.setFollowShop(cluePool.getOriginalShopId());
        task.setGroupId(cluePool.getGroupId());
        task.setBizId(cluePool.getBizId());
        return task;
    }

    /**
     * 创建跟进待办基本信息
     *
     * @param task
     * @return
     */
    protected FollowRecord createRecord(FollowTask task) {
        return createRecord(task, task.getBeginTime(), task.getDeadline());
    }

    protected FollowRecord createRecord(FollowTask task, LocalDateTime beginTime, LocalDateTime deadline) {
        FollowRecord record = new FollowRecord();
        record.setTaskId(task.getId());
        record.setBizType(task.getBizType());
        record.setType(task.getType());
        record.setCustomerId(task.getCustomerId());
        record.setCustomerName(task.getCustomerName());
        record.setMemberId(task.getMemberId());
        record.setUserId(task.getFollowUser());
        record.setUserName(task.getFollowUserName());
        record.setPlanTime(beginTime);
        record.setDeadline(deadline);
        record.setOutTime(OutTimeEnum.ONGOING);
        record.setShopId(task.getFollowShop());
        record.setGroupId(task.getGroupId());
        record.setSecondary(Boolean.TRUE.equals(task.getRedistribution()));
        record.setAddTodo(Boolean.FALSE);
        record.setTodoCode(getTodoCode());
        record.setBizId(task.getBizId());
        record.setPlateNo(task.getPlateNo());
        boolean compelTel = false;
        //2022-05-20 需求 集客潜客  跟进记录每隔两次根据记录必须打电话
        if (FollowTypeEnum.CF == task.getType() || FollowTypeEnum.PF == task.getType()) {
            if (followRecordService.checkCompelCall(task.getId(), task.getFollowUser(), 2)) {
                compelTel = Boolean.TRUE;
            }
            record.setCompelTel(compelTel);
        } else {
            // 其他跟进首次必须打电话
            long count = followRecordLogService.count(Wrappers.<FollowRecordLog>lambdaQuery()
                    .eq(FollowRecordLog::getTaskId, record.getTaskId())
                    .eq(FollowRecordLog::getAttType, AttTypeEnum.SMART_PHONE)
            );
            record.setCompelTel(count <= 0);
        }
        //同一天多个市场活动跟进客户后，视为多个活动已跟进
        //同一天同一潜客：跟进任一次视为后续跟进为已跟进
        if (FollowTypeEnum.PF.equals(task.getType()) || FollowTypeEnum.AF.equals(task.getType())) {
            List<FollowRecord> existList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                    .eq(FollowRecord::getCustomerId, record.getCustomerId())
                    .eq(FollowRecord::getType, record.getType())
                    .eq(FollowRecord::getUserId, record.getUserId())
                    .eq(FollowRecord::getOutTime, OutTimeEnum.BE_COMPLETE)
                    .eq(FollowRecord::getYn, Boolean.TRUE)
                    .ge(FollowRecord::getFollowTime, LocalDate.now().atStartOfDay())
                    .le(FollowRecord::getFollowTime, LocalDateTime.now())
            );
            if (!CollectionUtils.isEmpty(existList)) {
                record.setOutTime(OutTimeEnum.BE_SYSTEM);
                record.setFollowTime(LocalDateTime.now());
            }
        }
        return record;
    }

    /**
     * 处理对应跟进任务的待办
     *
     * @param taskId
     */
    @Transactional(rollbackFor = Exception.class)
    public void completeTodo(Long taskId) {
        final LocalDateTime now = LocalDateTime.now();
        followRecordService.removeByTaskId(taskId);
        List<FollowRecord> list = Optional.ofNullable(followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getTaskId, taskId)
                .eq(FollowRecord::getOutTime, OutTimeEnum.ONGOING)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .isNull(FollowRecord::getFollowTime)
                .ge(FollowRecord::getDeadline, now)
        )).orElse(new ArrayList<>());
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (FollowRecord record : list) {
            BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), String.valueOf(record.getId()), DateUtil.toDate(record.getPlanTime()), record.getShopId());
            record.setFollowTime(now);
            record.setOutTime(OutTimeEnum.BE_COMPLETE);
            boolean hasFollow = followRecordLogService.count(Wrappers.<FollowRecordLog>lambdaQuery()
                    .eq(FollowRecordLog::getRecordId, record.getId())
            ) > 0;

            if (hasFollow) {
                todoRpcService.complete(dto);
            } else {
                record.setOutTime(OutTimeEnum.BE_SYSTEM);
                todoRpcService.cancel(dto);
            }
        }
        followRecordService.updateBatchById(list);

        for (FollowRecord record : list) {
            eventPublisher.publishEvent(new RecordCompleteEvent(record, null));
        }
    }

    protected void updateContacts(FollowRecord record) {
        Long memberId = record.getMemberId();
        MemberUserDTO user = memberRpcService.user(memberId);
        if (Objects.nonNull(user)) {
            record.setContacts(user.getPhone());
        }
    }

    protected <T extends FollowDetailVO> void assembleRecordDetail(T vo, FollowRecord record, CluePool cluePool) {
        vo.setId(record.getId());
        vo.setCustomerId(record.getCustomerId());
        vo.setCustomerName(record.getCustomerName());
        vo.setType(record.getType().getValue());
        vo.setBizType(record.getBizType().getValue());
        queryTimes(cluePool.getId(), vo);
        vo.setTaskId(record.getTaskId());
        vo.setDeadline(record.getDeadline());
        vo.setUserId(record.getUserId());
        vo.setDetailId(cluePool.getRefererId());
        vo.setMobile(record.getContacts());
        vo.setRealMobile(record.getContacts());
        vo.setRegion(MobileUtil.attribution(record.getContacts()));

        //默认已经打过电话
        Boolean hadCall = Boolean.TRUE;
        if (Boolean.TRUE.equals(record.getCompelTel())) {
            long count = followRecordLogService.count(Wrappers.<FollowRecordLog>lambdaQuery()
                    .eq(FollowRecordLog::getTaskId, record.getTaskId())
                    .eq(FollowRecordLog::getAttType, AttTypeEnum.SMART_PHONE)
            );
            if (count <= 0) {
                hadCall = Boolean.FALSE;
            }
        }
        //是否已经打过电话
        vo.setHadCall(hadCall);
    }

    protected <T extends FollowDetailVO> void queryTimes(Long clueId, T vo) {
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getClueId, clueId)
        );
        if (CollectionUtils.isEmpty(list)) {
            vo.setTimes(0);
            return;
        }
        int sum = list.stream().filter(r -> Objects.nonNull(r.getTimes())).mapToInt(FollowTask::getTimes).sum();
        vo.setTimes(sum);
    }

    /**
     * 更新反馈
     *
     * @param taskId
     * @param feedbackTypeEnum
     */
    protected void feedbackTask(Long taskId, FeedbackTypeEnum feedbackTypeEnum) {
        FollowTask task = followTaskService.getById(taskId);
        if (Objects.isNull(task) || Objects.isNull(feedbackTypeEnum)) {
            return;
        }
        String feedback = task.getFeedback();
        Set<String> feedbackSet = StringUtils.isEmpty(feedback) ? new HashSet<>() : new HashSet<>(Arrays.asList(feedback.split(",")));
        feedbackSet.add(String.valueOf(feedbackTypeEnum.getValue()));
        task.setFeedback(String.join(",", feedbackSet));
        task.setLastFeedback(feedbackTypeEnum);
        followTaskService.updateById(task);
    }


    /**
     * 处理重复的线索
     *
     * @param cluePool 当前线索
     * @param strategy
     * @return 是否包含重复的活动
     */
    protected boolean dealRepeatClue(CluePool cluePool, FollowStrategy strategy) {
        //判断是否可以去重
        if (null == cluePool.getCoverFlag() || Boolean.FALSE.equals(cluePool.getCoverFlag())) {
            return false;
        }
        boolean isContainAF = false;
        //查询到重复的正在进行中线索
        //2022-5-30 活动邀约优先级高 不可处理
        List<CluePool> cluePoolList = cluePoolService.list(Wrappers.<CluePool>lambdaQuery()
                .eq(CluePool::getBizType, cluePool.getBizType())
                .eq(CluePool::getOriginalUserId, cluePool.getOriginalUserId())
                .eq(CluePool::getCustomerId, cluePool.getCustomerId())
                .in(CluePool::getClueStatus, ClueStatusEnum.ONGOING)
                .ne(CluePool::getId, cluePool.getId())
        );
        //查询重复的正在进行中任务
        if (!CollectionUtils.isEmpty(cluePoolList)) {
            //获取有时间交集的线索id
            List<Long> clueIds = new ArrayList<>();
            for (CluePool pool : cluePoolList) {
                //判断时间交集
                if (DateUtil.isInterSection(cluePool.getStartTime(), cluePool.getDeadline(), pool.getStartTime(), pool.getDeadline())) {
                    //判断重复的线索 是否是活动
                    if (FollowTypeEnum.AF == pool.getClueType() && FollowTypeEnum.AF != cluePool.getClueType()) {
                        isContainAF = true;
                    } else {
                        clueIds.add(pool.getId());
                    }
                }
            }
            if (CollectionUtil.isNotEmpty(clueIds)) {
                List<FollowTask> taskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                        .in(FollowTask::getClueId, clueIds)
                        .in(FollowTask::getState, TaskStateEnum.ONGOING)
                );
                if (null != taskList && !taskList.isEmpty()) {
                    //获取正在进行中的跟进记录
                    List<Long> taskIds = taskList.stream().map(FollowTask::getId).collect(Collectors.toList());
                    List<FollowRecord> followRecordList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                            .in(FollowRecord::getTaskId, taskIds)
                            .eq(FollowRecord::getYn, Boolean.TRUE)
                            .eq(FollowRecord::getOutTime, OutTimeEnum.ONGOING)
                    );
                    //调用已有的方法 分别处理
                    if (null != followRecordList && !followRecordList.isEmpty()) {
                        //需求：已经推送的完成  未推送的就逻辑删除
                        List<FollowRecord> updateList = new ArrayList<>();
                        for (FollowRecord record : followRecordList) {
                            if (Boolean.TRUE.equals(record.getAddTodo())) {
                                //已经推送到代办
                                strategy.complete(record, cluePool.getOriginalUserId(), true);
                            } else {
                                //未推送到代办
                                FollowRecord updateObj = new FollowRecord();
                                updateObj.setId(record.getId());
                                updateObj.setYn(false);
                                updateList.add(updateObj);
                            }
                        }
                        if (!updateList.isEmpty()) {
                            //逻辑删除
                            followRecordService.updateBatchById(updateList);
                        }
                    }
                }
            }
        }
        return isContainAF;
    }

    /**
     * 处理重复的跟进记录
     *
     * @param task
     */
    protected void dealRepeatRecord(FollowTask task) {
        List<FollowRecord> followRecordList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getTaskId, task.getId())
                .eq(FollowRecord::getType, task.getType())
                .eq(FollowRecord::getUserId, task.getFollowUser())
                .eq(FollowRecord::getCustomerId, task.getCustomerId())
                .eq(FollowRecord::getOutTime, OutTimeEnum.ONGOING)
                .eq(FollowRecord::getAddTodo, Boolean.FALSE)
                .eq(FollowRecord::getYn, Boolean.TRUE)
        );
        if (!CollectionUtils.isEmpty(followRecordList)) {
            followRecordList.forEach(r -> r.setYn(Boolean.FALSE));
            //将重复的跟进记录逻辑删除
            followRecordService.updateBatchById(followRecordList);
        }
    }
}
