package cn.fw.shirasawa.service.bus.follow.strategy;

import cn.fw.shirasawa.domain.db.OriginalData;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.pool.CluePool;
import cn.fw.shirasawa.domain.dto.FollowAttachmentDTO;
import cn.fw.shirasawa.domain.dto.FollowTaskCompleteDTO;
import cn.fw.shirasawa.domain.dto.FollowTerminationDTO;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.domain.vo.follow.FollowDetailVO;
import cn.fw.shirasawa.domain.vo.follow.FollowRecordHistoryVO;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author : kurisu
 * @className : FollowStrategy
 * @description : 跟进策略接口
 * @date: 2020-08-14 10:10
 */
public interface FollowStrategy {
    /**
     * 获取跟进类型
     *
     * @return
     */
    FollowTypeEnum getFollowType();

    /**
     * 待办编码
     *
     * @return
     */
    String getTodoCode();

    /**
     * 元数据生成任务
     *
     * @param originalData
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    boolean origin2task(OriginalData originalData) throws Exception;

    /**
     * 开始线索
     *
     * @param cluePool
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    void startClue(CluePool cluePool) throws Exception;

    /**
     * 跟进待办逾期
     *
     * @param record
     */
    @Transactional(rollbackFor = Exception.class)
    void overdue(FollowRecord record);

    /**
     * 完成跟进
     *
     * @param record 跟进待办
     * @param userId 完成人
     * @param ignoreCheck 是否忽略检查  true-忽略  false-不忽略
     */
    @Transactional(rollbackFor = Exception.class)
    void complete(FollowRecord record, Long userId,Boolean ignoreCheck);

    /**
     * 完成跟进
     *
     * @param record 跟进待办
     * @param userId 完成人
     */
    @Transactional(rollbackFor = Exception.class)
    void complete(FollowRecord record, Long userId);

    /**
     * 售前生成新的待办
     *
     * @param task
     * @param nextTime
     * @param nextEndTime
     * @param note
     */
    @Transactional(rollbackFor = Exception.class)
    void createNewRecord(FollowTask task, LocalDateTime nextTime, LocalDateTime nextEndTime, String note);

    /**
     * 完成跟进任务
     *
     * @param taskCompleteDTO
     */
    @Transactional(rollbackFor = Exception.class)
    void completeTask(FollowTaskCompleteDTO taskCompleteDTO);

    /**
     * 终止跟进任务
     *
     * @param terminationDTO
     */
    void terminationTaskPlan(FollowTerminationDTO terminationDTO);

    /**
     * 根据线索id结束正在进行中的跟进任务
     *
     * @param cluePool
     */
    @Transactional(rollbackFor = Exception.class)
    void endTask(CluePool cluePool);

    /**
     * 结束任务
     *
     * @param task
     * @param fromFlow 来自审批
     */
    @Transactional(rollbackFor = Exception.class)
    void endTask(FollowTask task, boolean fromFlow);

    /**
     * 查询跟进详情
     *
     * @param record
     * @return
     */
    FollowDetailVO queryDetail(FollowRecord record);

    /**
     * 查询跟进记录
     *
     * @param record
     * @return
     */
    List<FollowRecordHistoryVO> queryHistoryByRecord(FollowRecord record);

    /**
     * 查询跟进记录
     *
     * @param task
     * @return
     */
    List<FollowRecordHistoryVO> queryHistoryByTask(FollowTask task);

    /**
     * 上传跟进附件
     *
     * @param dto
     * @param record
     * @param userId
     */
    @Transactional(rollbackFor = Exception.class)
    void uploadAtt(FollowAttachmentDTO dto, FollowRecord record, Long userId);

    /**
     * 根据线索查询跟进历史记录
     *
     * @param cluePool
     * @return
     */
    List<FollowRecordHistoryVO> getRecordListByClue(CluePool cluePool);

    /**
     * 查询跟进池详情
     *
     * @param task
     * @return
     */
    FollowDetailVO followPoolDetail(FollowTask task);
}
