package cn.fw.shirasawa.service.bus.follow.strategy.impl;

import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.domain.db.OriginalData;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.pool.CluePool;
import cn.fw.shirasawa.domain.enums.ClueStatusEnum;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.domain.vo.follow.ACDetailVO;
import cn.fw.shirasawa.domain.vo.follow.FMDetailVO;
import cn.fw.shirasawa.domain.vo.follow.FollowDetailVO;
import cn.fw.shirasawa.rpc.valhalla.ValhallaRpcService;
import cn.fw.shirasawa.rpc.valhalla.dto.ValhallaCustomerDTO;
import cn.fw.shirasawa.service.bus.follow.strategy.AbstractFollowStrategy;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : FMFollowStrategy
 * @description : 首保策略
 * @date: 2020-08-17 10:46
 */
@Slf4j
@Component
public class FMFollowStrategy extends AbstractFollowStrategy {
    @Value("${follow.todo.FMCode}")
    @Getter
    private String FMCode;
    private final ValhallaRpcService valhallaRpcService;

    @Autowired
    public FMFollowStrategy(final ValhallaRpcService valhallaRpcService) {
        this.valhallaRpcService = valhallaRpcService;
    }

    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.FM;
    }

    @Override
    public String getTodoCode() {
        return getFMCode();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean origin2task(OriginalData originalData) throws Exception {
        List<CluePool> list = cluePoolService.list(Wrappers.<CluePool>lambdaQuery()
                .eq(CluePool::getClueType, getFollowType())
                .eq(CluePool::getFrameNo, originalData.getFrameNo())
                .eq(CluePool::getRefererId, originalData.getDetailId())
                .eq(CluePool::getGroupId, originalData.getGroupId())
                .in(CluePool::getClueStatus, ClueStatusEnum.WAITING)
        );

        if (!CollectionUtils.isEmpty(list)) {
            List<Long> needDelete = list.stream().map(CluePool::getId).collect(Collectors.toList());
            cluePoolService.removeByIds(needDelete);
        }
        int count = cluePoolService.count(Wrappers.<CluePool>lambdaQuery()
                .eq(CluePool::getClueType, getFollowType())
                .eq(CluePool::getFrameNo, originalData.getFrameNo())
                .eq(CluePool::getRefererId, originalData.getDetailId())
                .eq(CluePool::getGroupId, originalData.getGroupId())
                .in(CluePool::getClueStatus, ClueStatusEnum.ONGOING)
        );
        if (count > 0) {
            return true;
        }

        CluePool cluePool = transformClue(originalData);
        cluePool.setStartTime(originalData.getGenerateTime());
        LocalDateTime deadLine = Optional.ofNullable(originalData.getDeadline()).orElse(LocalDate.now().plusMonths(2L).atStartOfDay());
        cluePool.setDeadline(deadLine.minusSeconds(1L));
        cluePool.setBizId(originalData.getFrameNo());
        return cluePoolService.save(cluePool);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void startClue(CluePool cluePool) throws Exception {
        FollowTask task = this.createTask(cluePool);
        boolean saveTask = followTaskService.save(task);
        BV.isTrue(saveTask, () -> "生成跟进任务失败");
        FollowRecord record = createRecord(task, task.getBeginTime(), cluePool.getFirstDeadline());
        record.setNote(cluePool.getNote());
        record.setContacts(cluePool.getContacts());
        followRecordService.save(record);
        cluePool.setClueStatus(ClueStatusEnum.ONGOING);
        cluePoolService.updateById(cluePool);
    }


    @Override
    public FMDetailVO queryDetail(FollowRecord record) {
        FollowTask followTask = followTaskService.getById(record.getTaskId());
        BV.notNull(followTask, "跟进任务不存在");
        CluePool cluePool = cluePoolService.getById(followTask.getClueId());
        BV.notNull(cluePool, "跟进线索不存在");
        FMDetailVO vo = new FMDetailVO();
        assembleRecordDetail(vo, record, cluePool);
        vo.setTaskDeadline(followTask.getDeadline());
        vo.setFMExpiration(cluePool.getDeadline());
        ValhallaCustomerDTO dto = valhallaRpcService.queryByVin(record.getBizId(), record.getGroupId());
        if (Objects.nonNull(dto)) {
            vo.setCustomerId(dto.getId());
            vo.setPlateNo(dto.getPlateNo());
            vo.setAdviserName(dto.getAdviserName());
            vo.setBuyDate(DateUtil.date2LocalDateTime(dto.getBuyDate()));
        }
        vo.setVin(record.getBizId());
        return vo;
    }

    @Override
    public FMDetailVO followPoolDetail(FollowTask task) {
        CluePool cluePool = cluePoolService.getById(task.getClueId());
        BV.notNull(cluePool, "跟进线索不存在");
        FMDetailVO vo = new FMDetailVO();
        vo.setCustomerId(task.getCustomerId());
        vo.setCustomerName(task.getCustomerName());
        vo.setPlateNo(cluePool.getPlateNo());
        vo.setVin(cluePool.getBizId());
        vo.setType(task.getType().getValue());
        vo.setBizType(cluePool.getBizType().getValue());
        vo.setTaskId(task.getId());
        vo.setDeadline(task.getDeadline());
        queryTimes(cluePool.getId(), vo);
        ValhallaCustomerDTO dto = valhallaRpcService.queryByVin(cluePool.getBizId(), cluePool.getGroupId());
        if (Objects.nonNull(dto)) {
            vo.setCustomerId(dto.getId());
            vo.setCustomerName(dto.getName());
        }
        return vo;
    }
}
