package cn.fw.shirasawa.service.bus.follow.strategy.impl;

import cn.fw.shirasawa.domain.db.OriginalData;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.pool.CluePool;
import cn.fw.shirasawa.domain.enums.ClueStatusEnum;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.domain.enums.OutTimeEnum;
import cn.fw.shirasawa.service.bus.follow.strategy.AbstractFollowStrategy;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : PFFollowStrategy
 * @description : 潜客跟进
 * @date: 2022-01-11 15:37
 */
@Slf4j
@Component
@SuppressWarnings("Duplicates")
public class PFFollowStrategy extends AbstractFollowStrategy {
    @Value("${follow.todo.PFCode}")
    @Getter
    private String PFCode;

    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.PF;
    }

    @Override
    public String getTodoCode() {
        return getPFCode();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean origin2task(OriginalData originalData) throws Exception {
        List<CluePool> list = cluePoolService.list(Wrappers.<CluePool>lambdaQuery()
                .eq(CluePool::getClueType, getFollowType())
                .eq(CluePool::getCustomerId, originalData.getCustomerId())
                .eq(CluePool::getGroupId, originalData.getGroupId())
                .in(CluePool::getClueStatus, ClueStatusEnum.ONGOING, ClueStatusEnum.WAITING)
        );
        if (!CollectionUtils.isEmpty(list)) {
            return true;
        }
        CluePool cluePool = transformClue(originalData);
        cluePool.setStartTime(originalData.getGenerateTime());
        if (null != cluePool.getDeadline()) {
            cluePool.setDeadline(cluePool.getDeadline());
        } else {
            cluePool.setDeadline(LocalDate.now().plusYears(1L).atStartOfDay().minusSeconds(1L));
        }
        return cluePoolService.save(cluePool);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void startClue(CluePool cluePool) throws Exception {
        //处理重复的跟进业务
        boolean isContainAF = dealRepeatClue(cluePool, this);
        FollowTask task = this.createTask(cluePool);
        boolean saveTask = followTaskService.save(task);
        BV.isTrue(saveTask, () -> "生成跟进任务失败");
        FollowRecord record = createRecord(task);
        if (isContainAF) {
            //如果已经包含活动 直接进行删除  从而不会推待办
            record.setYn(Boolean.FALSE);
        }
        record.setNote(cluePool.getNote());
        record.setContacts(cluePool.getContacts());
        record.setDeadline(cluePool.getFirstDeadline());
        followRecordService.save(record);
        cluePool.setClueStatus(ClueStatusEnum.ONGOING);
        cluePoolService.updateById(cluePool);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createNewRecord(FollowTask task, LocalDateTime nextTime, LocalDateTime nextEndTime, String note) {
        if (Objects.isNull(nextTime)) {
            return;
        }
        //处理重复的跟进记录
        dealRepeatRecord(task);
        long count = followRecordService.count(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getTaskId, task.getId())
                .eq(FollowRecord::getUserId, task.getFollowUser())
                .eq(FollowRecord::getOutTime, OutTimeEnum.ONGOING)
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .isNull(FollowRecord::getFollowTime)
        );
        BV.isZero(count, () -> "存在进行中的待办，请先处理");

        FollowRecord record = this.createRecord(task, nextTime, nextEndTime);
        updateContacts(record);
        record.setNote(note);
        followRecordService.save(record);
    }

}
